#ifndef __HTTP_H
#define __HTTP_H

#include "qobject.h"
#include "qstring.h"
#include <queue>
#include <pthread.h>

struct _HTRequest;
struct _HTList;
struct _HTChunk;

typedef _HTRequest HTRequest;
typedef _HTList HTList;
typedef _HTChunk HTChunk;

class KHTMLWidget;

class HTTPConnection : public QObject
{
  Q_OBJECT;
protected:
  HTRequest * request;
  HTList * converters;
  HTList * encodings;

  pthread_mutex_t m_ListLock;

public:

  class HTTPRequest
    {
    protected:
      QString m_strURL;
      unsigned char * m_Buffer;
      int m_nSize;
      bool m_bNewPage;
    public:
      HTTPRequest(const QString & url, bool bNewPage=false) 
	{m_strURL = url; m_bNewPage = bNewPage; m_nSize = 0; m_Buffer = 0; }
      ~HTTPRequest();
      const char * getURL() const { return m_strURL; }
      unsigned char * getBuffer() { return m_Buffer; }
      bool newPage() { return m_bNewPage; }
      int getSize() { return m_nSize; }
      void addBuffer(unsigned char * buf, int len);
    };


  HTTPConnection();
  ~HTTPConnection();
  int Connect(QObject * p);

  void loadPage(const char * URL);
  void setHTML(KHTMLWidget * p);
  KHTMLWidget * HTML() const { return m_pHTML; }

 protected:
  queue<HTTPRequest *> m_Reqs;
  queue<HTTPRequest *> m_Finished;
  pthread_t m_Thread;
  bool m_bThreadRunning;
  KHTMLWidget * m_pHTML;

  void StartThread();
  void StopThread();
  void BeginDataTimer();
  static void * runthread(void* p);

 public:

  void AddReq(const char * url, bool bNewPage=false);
  HTTPRequest * GetReq();
  int ReqSize();

  void AddFinished(HTTPRequest * p);
  HTTPRequest * GetFinished();
  int FinishedSize();

  int getURL(HTTPRequest * r);
  
  protected slots:
    void slotOpenURL(const char * URL);
  void slotClick(const char * d1, int b, const char * p);
};

#endif
