
#include <kapp.h>
#include "html.h"
#include <Fl.H>
#include <fl_draw.H>
#include <Fl_Window.H>
#include <Fl_Button.H>
#include <Fl_Pixmap.H>
#include <Fl_Input.H>
#include <Fl_Pack.H>
#include "http.h"
#include "history.h"

extern Fl_Button *Forward;
extern Fl_Button *Backward;
extern Fl_Button *Stop;
extern Fl_Button *Reload;

Fl_Window* make_window();

const QColor color0(0,0,0);
const QColor color1(255,255,255);
const QColor black(0,0,0);
const QColor white(255,255,255);
const QColor darkGray(64,64,64);
const QColor gray(128,128,128);
const QColor lightGray(178,178,178);
const QColor red(255,0,0);
const QColor green(0,255,0);
const QColor blue(0,0,255);
const QColor cyan(128,0,128);
const QColor magenta(128,128,0);
const QColor yellow(0,128,128);
const QColor darkRed(178,64,64);
const QColor darkGreen(64,178,64);
const QColor darkBlue(64,64,178);
const QColor darkCyan(178,64,178);
const QColor darkMagenta(178,178,64);
const QColor darkYellow(64,178,178);

KApplication g_KApp;

const QCursor arrowCursor;      
const QCursor upArrowCursor;    
const QCursor crossCursor;      
const QCursor waitCursor;       
const QCursor ibeamCursor;      
const QCursor sizeVerCursor;    
const QCursor sizeHorCursor;    
const QCursor sizeBDiagCursor;  
const QCursor sizeFDiagCursor;  
const QCursor sizeAllCursor;    
const QCursor blankCursor;      


#define xHTML_WIDGET KHTMLView
#define HTML_WIDGET KHTMLWidget


HTML_WIDGET * g_HTML;
HTTPConnection * g_Conn;

#ifdef _NANOX

// the below is needed to fix some X compilation problems with Nano-X
unsigned long KeyPress;
bool XCheckTypedEvent( Display *, unsigned long , 
		       XEvent * )
{
	return false;
}
#endif

int main(int argc, char ** argv)
{

  Fl_Window * mw;

  g_Conn = new HTTPConnection();

  mw = make_window();

  g_Conn->setHTML(g_HTML);

  mw->show(argc,argv);

  g_HTML->setGeometry(0,0,640,450);
  
  while(Fl::wait()) {}

  return 0;
}

Fl_Button *Forward=(Fl_Button *)0;


static unsigned char *image_foo[] = {
(unsigned char*)
"21 22 10 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #838183",
(unsigned char*)
"@\tc #00FF00",
(unsigned char*)
"#\tc #008183",
(unsigned char*)
"$\tc #FFFFFF",
(unsigned char*)
"%\tc #008200",
(unsigned char*)
"&\tc #003439",
(unsigned char*)
"*\tc #202020",
(unsigned char*)
"=\tc #4199A4",
(unsigned char*)
".....................",
(unsigned char*)
"........+@...........",
(unsigned char*)
"........#$@..........",
(unsigned char*)
".......+%.$@.........",
(unsigned char*)
".......&%@.$@........",
(unsigned char*)
"......+&%.@.$@.......",
(unsigned char*)
"......#*%@.@.$@......",
(unsigned char*)
".....+**%.@.@.$@.....",
(unsigned char*)
".....&.@.@.@.@.$@....",
(unsigned char*)
"....&#@.@.@.@.@.$@...",
(unsigned char*)
"....**.@.@.@.@.@.$=..",
(unsigned char*)
"....**@.@.@.@.@.@$=..",
(unsigned char*)
"....&*###########%...",
(unsigned char*)
"....&*%%%%%#%#%=%....",
(unsigned char*)
"....#&%%%%%%=%=%.....",
(unsigned char*)
"....+&%%%%%=%=%......",
(unsigned char*)
".....****%%%=%.......",
(unsigned char*)
".++++#&&&%%=%+++.....",
(unsigned char*)
"+++++++&&%=&+++......",
(unsigned char*)
".++++++&&=&++........",
(unsigned char*)
"....++++&&++.........",
(unsigned char*)
"....................."
};
static Fl_Pixmap pixmap_foo(image_foo);

Fl_Button *Backward=(Fl_Button *)0;

static unsigned char *image_back[] = {
(unsigned char*)
"21 22 10 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #00FF00",
(unsigned char*)
"@\tc #838183",
(unsigned char*)
"#\tc #FFFFFF",
(unsigned char*)
"$\tc #008183",
(unsigned char*)
"%\tc #008200",
(unsigned char*)
"&\tc #003439",
(unsigned char*)
"*\tc #202020",
(unsigned char*)
"=\tc #4199A4",
(unsigned char*)
".....................",
(unsigned char*)
"...........+@........",
(unsigned char*)
"..........+#$........",
(unsigned char*)
".........+#.%@.......",
(unsigned char*)
"........+#.+%&.......",
(unsigned char*)
".......+#.+.%&@......",
(unsigned char*)
"......+#.+.+%*$......",
(unsigned char*)
".....+#.+.+.%**@.....",
(unsigned char*)
"....+#.+.+.+.+.&.....",
(unsigned char*)
"...+#.+.+.+.+.+$&....",
(unsigned char*)
"..=#.+.+.+.+.+.**....",
(unsigned char*)
"..=#+.+.+.+.+.+**....",
(unsigned char*)
"...%$$$$$$$$$$$*&....",
(unsigned char*)
"....%=%$%$%%%%%*&....",
(unsigned char*)
".....%=%=%%%%%%&$....",
(unsigned char*)
"......%=%=%%%%%&@....",
(unsigned char*)
".......%=%%%****.....",
(unsigned char*)
".....@@@%=%%&&&$@@@@.",
(unsigned char*)
"......@@@&=%&&@@@@@@@",
(unsigned char*)
"........@@&=&&@@@@@@.",
(unsigned char*)
".........@@&&@@@@....",
(unsigned char*)
"....................."
};
static Fl_Pixmap pixmap_back(image_back);

Fl_Button *Stop=(Fl_Button *)0;

static unsigned char *image_stop[] = {
(unsigned char*)
"21 21 10 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #6265CD",
(unsigned char*)
"@\tc #838183",
(unsigned char*)
"#\tc #9C99FF",
(unsigned char*)
"$\tc #313062",
(unsigned char*)
"%\tc #000000",
(unsigned char*)
"&\tc #830000",
(unsigned char*)
"*\tc #FF0000",
(unsigned char*)
"=\tc #202020",
(unsigned char*)
".....................",
(unsigned char*)
"....+++++++++++@.....",
(unsigned char*)
"...#@@#@#@#@#@+$.....",
(unsigned char*)
".$#+##@$%$@.@.@%%%$..",
(unsigned char*)
".$.+.@&&&&%@#.$@+%$..",
(unsigned char*)
"..$+.&&*&&$%@.@+%&%..",
(unsigned char*)
"..$+..***&$%#.+.&&...",
(unsigned char*)
"...+...**&$%@.@$&&...",
(unsigned char*)
"...+.#.*&=%=#.+$.&...",
(unsigned char*)
".$@+.##====+@.$$$$$..",
(unsigned char*)
".$#+.#%%%%+$#.+@+%$..",
(unsigned char*)
"..$+.@$$++$$@.@+%$$..",
(unsigned char*)
"...+.%@@$$$$#.+.$$...",
(unsigned char*)
"...+.%@@$$=@@.@$$$...",
(unsigned char*)
".$#+.#@$$$$#@.@$$$%..",
(unsigned char*)
".$#+.@%%%%=@#.$@+%$..",
(unsigned char*)
".@$+.%====%$@.++%$$..",
(unsigned char*)
"..$+.%$$$=$%#.+@$$...",
(unsigned char*)
"...+..$$=$=%@.+$$$@@.",
(unsigned char*)
"...+###===$@#@+=@@@@.",
(unsigned char*)
".....$$$+++++$+%@@@.."
};
static Fl_Pixmap pixmap_stop(image_stop);

Fl_Button *Reload=(Fl_Button *)0;

static unsigned char *image_reload[] = {
(unsigned char*)
"21 21 9 1",
(unsigned char*)
" \tc None",
(unsigned char*)
".\tc #C5C2C5",
(unsigned char*)
"+\tc #6265CD",
(unsigned char*)
"@\tc #313062",
(unsigned char*)
"#\tc #FFFFFF",
(unsigned char*)
"$\tc #9C99FF",
(unsigned char*)
"%\tc #202020",
(unsigned char*)
"&\tc #000083",
(unsigned char*)
"*\tc #838183",
(unsigned char*)
".....................",
(unsigned char*)
".....++++++++@.......",
(unsigned char*)
"....+#+$+$++%&@@.....",
(unsigned char*)
"...+#$$$$$+%%%&@@....",
(unsigned char*)
"#....$.$.$.$$$+%%@...",
(unsigned char*)
"+#..$.$.$.$.$%%%@@...",
(unsigned char*)
".+#..$.$.$.$%@&&@@...",
(unsigned char*)
"..+#......+%@&&&@@...",
(unsigned char*)
"...+#....+%@&&&&@@...",
(unsigned char*)
"....+#..+%@+&&&&@@...",
(unsigned char*)
".....+#+%@.+&&&&@@...",
(unsigned char*)
"......@@@..+&&&%@@...",
(unsigned char*)
"...........+&&&%@@.*.",
(unsigned char*)
"...........$$&%%@@.**",
(unsigned char*)
"...$$$$$$$$$+%&%@@***",
(unsigned char*)
"...$$$$$$$$$+%%%@@++.",
(unsigned char*)
"...$+$+$+$+$++%@+@+*.",
(unsigned char*)
"...$++++++++++++@+**.",
(unsigned char*)
"....$++@@@@@&@@@+**..",
(unsigned char*)
".......***********...",
(unsigned char*)
"....................."
};

static Fl_Pixmap pixmap_reload(image_reload);

//davet
#define MAX_PROTO_STRLEN 7 // must be set to length of longest protocol
static char *protocols[] = 
{
  "http://",
  "file://",
#ifdef OPTIONAL_URI_PROTOCOLS
  "ftp://",
  "afs://",
  "news:",
  "nntp:",
  "mid:",
  "cid:",
  "mailto:",
  "wais://",
  "prospero://",
  "telnet://"
  "gopher://",
#endif
  NULL
};

void LoadPage(const char * URL)
{
  char buf[1024];
  int pindex;

  // get URL protocol
  strncpy(buf,URL,MAX_PROTO_STRLEN);
  buf[MAX_PROTO_STRLEN] = 0x00;

  // check for protocol in list
  pindex = 0;
  while(protocols[pindex] != NULL)
  {
    if(strncmp(buf,protocols[pindex],strlen(protocols[pindex])) == 0)
      break;
    ++pindex;
  }

  // check for missing protocol
  if(protocols[pindex] == NULL)
    sprintf(buf,"http://%s",URL);
  else
    strcpy(buf,URL);

  g_Conn->loadPage(buf);
}

static char oldstr[256] = "";
static int hindex = 0;
void Input_Callback(Fl_Widget * w, void * data)
{
  Fl_Input * i = (Fl_Input*)w;
  QString str  =  i->value();
  History hobj;

  // if old and new strings match, must have been a CR
  if(strcmp((char *)str,oldstr) == 0)
  {
    oldstr[0] = 0x00;
    hobj.AddToHistoryList(str);
    hindex = 1;
    LoadPage(str);
  }

  // if there's a space in str, set a history entry
  else if(strstr((char *)str," "))
  {
    i->value(hobj.GetHistoryEntry(hindex));
    i->set_changed();
    i->mark(256);
    strcpy(oldstr,hobj.GetHistoryEntry(hindex));
    if(hobj.GetHistoryEntry(++hindex) == NULL)
      hindex = 0;
  }

  // must be an updated string - save it
  else
  {
    strcpy(oldstr,(char *)str);
  }
}


Fl_Window* make_window() 
{
  char buf[256];
  Fl_Window* w;
  { Fl_Window* o = new Fl_Window(640, 480);
    w = o;
    { 
      Fl_Button* o = Forward = new Fl_Button(30, 5, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      pixmap_foo.label(o);
    }
    { 
      Fl_Button* o = Backward = new Fl_Button(5, 5, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      pixmap_back.label(o);
    }
    { 
      Fl_Button* o = Stop = new Fl_Button(55, 5, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      pixmap_stop.label(o);
    }
    { 
      Fl_Button* o = Reload = new Fl_Button(80, 5, 25, 25);
      o->box(FL_OFLAT_BOX);
      o->down_box(FL_ROUND_DOWN_BOX);
      pixmap_reload.label(o);
    }
    { 
      Fl_Input * o = new Fl_Input(145, 5, 480, 25, "");
      o->callback(Input_Callback,0);
      o->when(FL_WHEN_ENTER_KEY_CHANGED);
      //o->value("http://www.viewml.com");
      //o->set_changed();
    }

    { 
      Fl_Pack* p = new Fl_Pack(5, 35, 640, 450); 

      HTML_WIDGET * v;
      v = new HTML_WIDGET();
      p->resizable(v);
      
      g_Conn->Connect(v);
      g_HTML = v;
      p->end();
      o->resizable(p);
    }

    o->end();
  }
  return w;
}

